(function($) {
	window.scheduleEditorIds = 0;

	var wdayNames = ['Monday', 'Tuesday', 'Wednsday', 'Thursday', 'Friday', 'Saturday', 'Sunday'];
	var wdayNamesShort = ['Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun'];

	$.fn.jqplotRemove = function() {
		return $(this).removeClass('jqplot-target').css('width', '').css('height', '').empty();
	};
	
	$.fn.scheduleEditor = function(schedule, updateFunction) {
		var WIDTH_EXPANDED = 300;
		var WIDTH_COLLAPSED = 100;
		var HEIGHT = 200;
		
		var TEMP_NO_FROST = 'frost_protection';
		var TEMP_NO_FROST_COLOUR = '#0000ff';
		var TEMP_ENERGY_SAVING = 'energy_saving';
		var TEMP_ENERGY_SAVING_COLOUR = '#40ff40';
		var TEMP_MIN = 6;
		var TEMP_MAX = 30;
		var TEMP_DEFAULT = 20;
		var MIN_TIME_RANGE = 10;
		var MIN_TIME_RANGE_MS = MIN_TIME_RANGE * 60 * 1000; // in 1ms units
		
		var schedEditor = this;
		var $schedEditor = $(this);
		var scheduleEditorId = scheduleEditorIds;
		
		// Helpers functions //

		function wDayGraphId(wday) {
			return '#wDay' + wday + '_' + scheduleEditorId;
		};

		// returns min and max temperature of the week schedule		
		function getMinMax(wd) {
			var arr = [];
			for (var wdi = 1; wdi <= 7; wdi++)
				if (!wd || wd == wdi)
					arr.push.apply(arr, schedule[wdi - 1].map(function(el) { return el[1]} ));
			arr = arr.filter(function(el) { return el != TEMP_NO_FROST && el != TEMP_ENERGY_SAVING});
			var sarr = arr.sort(function(a, b) { return a-b; });
			return [sarr[0], sarr[sarr.length - 1]];
		};

		// this is just to show label - not used in graph drawing
		function nextSwitchPointTime(wd, sp) {
			if ((sp + 1) in schedule[wd - 1])
				return schedule[wd - 1][sp + 1];
			else
				for (var i = 1; i <= 7; i++)
					if (0 in schedule[(wd - 1 + i) % 7])
						return ['24:00', schedule[(wd - 1 + i) % 7][0][1]];
		};
		
		// get last switchpoint before current day
		function prevSwitchPointToThisDay(wd) {
			for (i = 1; i <= 7; i++) {
				var prevDay = schedule[(wd - 1 - i + 7) % 7];
				if (prevDay.length)
					return prevDay[prevDay.length - 1];
			};
			return ['0:00', 20]; // default in case no points found
		};

		function nextWeekDay(wd) {
			return (wd == 7) ? 1 : (wd + 1);
		};
		
		function prevWeekDay(wd) {
			return (wd == 1) ? 7 : (wd - 1);
		};
		
		function copySchdule(wdFrom, wdTo) {
			if (wdFrom == wdTo)
				return;

			schedule[wdTo - 1] = [];
			for (var spi = 0; spi < schedule[wdFrom - 1].length; spi++) // copy of the array
				schedule[wdTo - 1][spi] = schedule[wdFrom - 1][spi].slice(); 
			updateScheduleNode();
			for (var wdi = 1; wdi <=7 ; wdi ++)
				renderWdayGraph(wdi, wdi == wdFrom);
		};
		
		function checkTime(input) {
			var wd = parseInt($schedEditor.find('#switchPoints').attr('day'), 10);
			var sp = parseInt($(input).closest('.switchPointData').attr('switchPoint'), 10);
			var val = $(input).val();

			return {
				minTime: (sp != 0) ? new Date($schedEditor.find('.switchPointData[switchPoint=' + (sp - 1) + '] .switchPoint').timeEntry('getTime').getTime() + MIN_TIME_RANGE_MS) : null,
				maxTime: ((sp + 1) in schedule[wd - 1]) ? new Date($schedEditor.find('.switchPointData[switchPoint=' + (sp + 1) + '] .switchPoint').timeEntry('getTime').getTime() - MIN_TIME_RANGE_MS) : null
			}
		};

		function dayTemperatureShift(wd, shift) {
			var spNum = schedule[wd - 1].length;
			for (var sp = 0; sp < spNum; sp++)
				if (schedule[wd - 1][sp][1] != TEMP_NO_FROST && schedule[wd - 1][sp][1] != TEMP_ENERGY_SAVING)
					schedule[wd - 1][sp][1] += parseInt(shift, 10);
			updateScheduleNode();
		};
		
		function updateScheduleNode() {
			updateFunction(schedEditor, schedule);
		};
		
		// Widgets functions //
		
		function expand(wday, showPoint) {
			$schedEditor.find(wDayGraphId(wday)).attr('expanded', '1');
			renderWdayGraph(wday, true, showPoint);
			fillSwitchPoints(wday, showPoint);
		};
		
		function collapse(wday) {
			$schedEditor.find(wDayGraphId(wday)).attr('expanded', '0');
			renderWdayGraph(wday, false);
		};
		
		function collapseExpand(wday, showPoint) {
			for (var wd = 1; wd <= 7; wd++) {
				if (wd != wday && $schedEditor.find(wDayGraphId(wd)).attr('expanded') == '1')
					collapse(wd);
			};
			
			if ($schedEditor.find(wDayGraphId(wday)).attr('expanded') == '1') {
				collapse(wday);
				fillSwitchPoints(null);
			} else
				expand(wday, showPoint);
		};

		function addSwitchPoint(sp, time, timeEnd, temp) {
			var row = $schedEditor.find('.switchPointData[switchPoint=' + sp + ']');
			row.find('.switchPoint').val(time)
				.timeEntry({show24Hours: true, spinnerImage: '', timeSteps: [1, MIN_TIME_RANGE, 0], beforeShow: checkTime})
				.unbind('.scheduleEditor')
				.bind('keydown.scheduleEditor', handleTimeKeyDown)
				.bind('change.scheduleEditor focus.scheduleEditor', updateInterface);
			row.find('.switchPointEnd').val(timeEnd);
			row.find('.switchPointTemp').val(temp)
				.unbind('.scheduleEditor')
				.bind('keydown.scheduleEditor', handleTempKeyDown)
				.bind('keyup.scheduleEditor', handleTempKeyUp)
				.bind('change.scheduleEditor focus.scheduleEditor', updateInterface);
			row.fadeIn()
			return row;
		};
		
		function hideSwitchPoint(sp) {
			return $schedEditor.find('.switchPointData[switchPoint=' + sp + ']').fadeOut();
		};

		function addFirstSwitchPoint(wd) {
			if (0 in schedule[wd - 1]) {
				if (schedule[wd - 1][0][0] == '0:00' || schedule[wd - 1][0][0] == '00:00')
					return hideSwitchPoint(-1);
				else
					return addSwitchPoint(-1, '0:00', schedule[wd - 1][0][0], prevSwitchPointToThisDay(wd)[1]);
			} else
				return addSwitchPoint(-1, '0:00', '24:00', prevSwitchPointToThisDay(wd)[1]);
		};
		
		function updateInterfaceOnChange(wd, sp, updateNext) {
			renderWdayGraph(wd, true, sp);
			if (updateNext || sp >= schedule[wd - 1].length - 1) {
				var iwd = wd;
				do {
					iwd = nextWeekDay(iwd);
					renderWdayGraph(iwd);
				} while (!schedule[iwd - 1].length && iwd != prevWeekDay(wd));  // exit on next graph with points or after a full round
			};
			addFirstSwitchPoint(wd);
		};
		
		function updateInterface(event) {
			var wd = parseInt($schedEditor.find('#switchPoints').attr('day'), 10);
			var sp = parseInt($(this).closest('.switchPointData').attr('switchPoint'), 10);
			var val = $(this).val();
			var vtype = ($(this).hasClass('switchPoint')) ? 0 : 1;
			
			if (event.type == 'change') {
				schedule[wd - 1][sp][vtype] = (vtype && val != TEMP_NO_FROST && val != TEMP_ENERGY_SAVING) ? parseInt(val, 10) : val;
				updateScheduleNode();
				updateInterfaceOnChange(wd, sp);
			} else
				renderWdayGraph(wd, true, sp);
		};
		
		function fillSwitchPoints(wd, focusPoint) {
			var sp = -1;

			if (wd !== null) {
				$schedEditor.find('#switchPoints').attr('day', wd);
				var ctrl = $schedEditor.find('#scheduleControl');
				var ctrlMargin = {'margin-left': $schedEditor.find('.dayGraph[expanded=1]').position().left - $schedEditor.find('.dayGraph.axisLeft').position().left - $schedEditor.find('.dayGraph.axisLeft').css('margin-left').replace('px', '') };
				if (ctrl.is(':visible'))
					ctrl.animate(ctrlMargin);
				else
					ctrl.css(ctrlMargin).slideDown();
			} else
				$schedEditor.find('#scheduleControl').slideUp();

			if ((wd - 1) in schedule) {
				addFirstSwitchPoint(wd).find('.appendSwitchPoint').fadeIn();
				sp = 0;
				for (; sp <= 8; sp++) {
					if (sp in schedule[wd - 1]) {
						addSwitchPoint(sp, schedule[wd - 1][sp][0], nextSwitchPointTime(wd, sp)[0], schedule[wd - 1][sp][1])
							.find('.appendSwitchPoint').fadeIn();
					} else
						break;
				}
				if (sp == 8)
					$schedEditor.find('.appendSwitchPoint').fadeOut();
			};
			for (; sp <= 8; sp++)
				hideSwitchPoint(sp);
			
			
			$schedEditor.find('.removeSwitchPoint').unbind('.scheduleEditor').bind('click.scheduleEditor', removeSwitchPoint);
			$schedEditor.find('.appendSwitchPoint').unbind('.scheduleEditor').bind('click.scheduleEditor', appendSwitchPoint);
			
			if (focusPoint !== undefined)
				$schedEditor.find('.switchPointData[switchPoint=' + focusPoint + '] .switchPoint').focus();
		};

		function collapseSwitchPoints() {
			collapseExpand($schedEditor.find('#switchPoints').attr('day'));
			$schedEditor.find('.dayGraph[day=' + $schedEditor.find('#switchPoints').attr('day') + ']').focus();
			renderWdayGraph($schedEditor.find('#switchPoints').attr('day'));
		};

		// Event handlers //

		function handleTempKeyUp(event) {
			var inp = $(event.target);
			switch (event.which) {
				case 38: // up
				case 40: // down
					inp.change();
					break;
			}
		};
		
		function handleCtrlRightLeftUpDownKeyUp(event) {
		/* Disable. Not finished yet.
			var inp = $(event.target);
			switch (event.which) {
				case 38: // up/down
				case 40:
					event.ctrlKey = false;
					if (inp.hasClass('switchPointTemp'))
						inp.trigger(event);
					else
						inp.parent().find('.switchPointTemp').trigger(event);
					event.stopImmediatePropagation(); // to prevent TimeEntry handler to capture this event
					event.preventDefault();
					break;
				
				case 37: // left/right
				case 39:
					event.ctrlKey = false;
					event.which += 1; // up/down
					event.keyCode += 1;
					if (inp.hasClass('switchPointTemp'))
						inp.parent().find('.switchPoint').trigger(event);
					else
						inp.trigger(event);
					event.stopImmediatePropagation(); // to prevent original behaviour of the combo box
					event.preventDefault();
					break;					
			};
		*/
		};

		function handleTempKeyDown(event) {
			var inp = $(event.target);
			switch (event.which) {
				case 37: // left/right
				case 39:
					if (event.ctrlKey)
						handleCtrlRightLeftUpDownKeyUp(event);
					event.preventDefault(); // disallow up/down actions on left/right press.
					break;
				
				case 38: // up/down
				case 40:
					if (event.ctrlKey)
						handleCtrlRightLeftUpDownKeyUp(event);
					break;

				case 9: // enter
					event.preventDefault();
				case 13: // tab
					if (event.shiftKey) {
						$(this).closest('.switchPointData').find('.switchPoint').focus();
					} else
						if ($(this).closest('.switchPointData').next(':visible').size())
							$(this).closest('.switchPointData').next().find('.switchPoint').focus();
						else
							collapseExpand(nextWeekDay(parseInt($schedEditor.find('#switchPoints').attr('day'), 10)), 0);
					break;

				case 27: // escape
					collapseSwitchPoints();
					break;
			};
		};
		
		function handleTimeKeyDown(event) {
			switch (event.which) {
				case 37: // left/right
				case 39:
					if (event.ctrlKey)
						handleCtrlRightLeftUpDownKeyUp(event);
					break;
				
				case 38: // up/down
				case 40:
					if (event.ctrlKey)
						handleCtrlRightLeftUpDownKeyUp(event);
					break;

				case 9: // tab
					event.preventDefault();
				case 13: // enter
					if (event.shiftKey) {
						if (parseInt($(this).closest('.switchPointData').attr('switchPoint'), 10) > 0)
							$(this).closest('.switchPointData').prev().find('.switchPointTemp').focus();
						else {
							var pwd = prevWeekDay(parseInt($schedEditor.find('#switchPoints').attr('day'), 10));
							collapseExpand(pwd, schedule[pwd - 1].length - 1);
						}
					} else
						$(this).closest('.switchPointData').find('.switchPointTemp').focus();
					break;

				case 27:
					collapseSwitchPoints();
					break;
			};
		};
		
		// Add/remove switch points //

		function removeSwitchPoint() {
			var wd = parseInt($schedEditor.find('#switchPoints').attr('day'), 10);
			var sp = parseInt($(this).closest('.switchPointData').attr('switchPoint'), 10);
			
			schedule[wd - 1].splice(sp, 1);
			updateScheduleNode();
			fillSwitchPoints(wd);
			updateInterfaceOnChange(wd, sp);
		};

		function appendSwitchPoint() {
			var wd = parseInt($schedEditor.find('#switchPoints').attr('day'), 10);
			var sp = parseInt($(this).closest('.switchPointData').attr('switchPoint'), 10);
			var setTime = new Date($(this).closest('.switchPointData').find('.switchPoint').timeEntry('getTime').getTime() + MIN_TIME_RANGE_MS);
			var nextInput = $(this).closest('.switchPointData').next().find('.switchPoint:visible');
			var nextTime = (nextInput.size()) ? nextInput.timeEntry('getTime').getTime() : null;
			var temperature = (sp > 0) ? schedule[wd - 1][sp][1] : TEMP_DEFAULT;
			
			if (nextTime && nextTime <= setTime)
				return;
			schedule[wd - 1].splice(sp + 1, 0, [setTime.getHours().toString(10) + ':' + setTime.getMinutes().toString(10), temperature]);
			updateScheduleNode();
			fillSwitchPoints(wd);
			updateInterfaceOnChange(wd, sp);
		};

		// Graphics drawing //
		
		function renderWdayGraph(weekDay, expand, sp, axisOnly) {
			var seriesColors = ["#4bb2c5"];
			var seriesParams = [ { lineWidth: 1, showMarker: false }];
			if (!axisOnly) {
				var points = [['2000-01-01 ' + '0:00:00', prevSwitchPointToThisDay(weekDay)[1]]];
				$.each(schedule[weekDay - 1], function(i, timeTemp) {
					points.push(['2000-01-01 ' + timeTemp[0] + ':00', points[points.length - 1][1]]);
					points.push(['2000-01-01 ' + timeTemp[0] + ':01', timeTemp[1]]);
				});
				points.push(['2000-01-02 ' + '0:00:00', points[points.length - 1][1]]);
				var title = (expand) ? wdayNames[weekDay - 1] : wdayNamesShort[weekDay - 1];
				var convasId = wDayGraphId(weekDay);
				var width = ((expand) ? WIDTH_EXPANDED : WIDTH_COLLAPSED) + 'px';
				var series = [points];
				
				// No Frost & Energy Saving modes
				for (var pi = 0; pi < points.length; pi+=2) {
					var colour = null;
					var temp = points[pi][1];
					if (temp == TEMP_NO_FROST)
						colour = TEMP_NO_FROST_COLOUR;
					if (temp == TEMP_ENERGY_SAVING)
						colour = TEMP_ENERGY_SAVING_COLOUR;
					if (colour) {
						series.push([[points[pi][0], TEMP_MIN], [points[pi+1][0], TEMP_MIN]]);
						seriesParams.push(seriesParams[0]);
						seriesColors.push(colour);
						points[pi][1] = TEMP_MIN;
						points[pi+1][1] = TEMP_MIN;
					}
				}

				// current point
				if (sp !== null && sp != undefined && sp in schedule[weekDay - 1]) {
					series.push([[points[2*(sp+1)][0], points[2*(sp+1)][1]]]);
					seriesParams.push({ showMarker: true });
					seriesColors.push('#ff7733');
				}
			} else {
				var points = [['2000-01-01 0:00', 0]];
				var title = '&nbsp;';
				var convasId = '#' + axisOnly + 'Axis' + '_' + scheduleEditorId;
				var width = '1px';
				var series = [points];
			};
			var $convas = $(convasId).jqplotRemove();

			$.jqplot.config.enablePlugins = true;
			$.jqplot(convasId.substring(1), series, {
				title: title,
				width: width,
				height: HEIGHT + 'px',
				series: seriesParams,
				
				seriesColors: seriesColors,
				
				grid: {
					borderWidth: 1
				},
				
				axes: {
					xaxis: {
						renderer: $.jqplot.DateAxisRenderer,
						tickOptions: {
							formatString: '%#H:%M',
							showMark: false
						},
						tickInterval: expand ? '3 hour' : '6 hour',
						min: '2000-01-01 0:00',
						max: '2000-01-02 0:00'
					},
					yaxis: (axisOnly) ? {
							ticks: [[5, 5], [10, 10], [15, 15], [20, 20], [25, 25], [30, 30]],
							tickOptions: {
								showMark: false,
								showGridline: false
							}
						} : {
							ticks: [[5, 5], [10, 10], [15, 15], [20, 20], [25, 25], [30, 30]],
							showTicks: false
						}
				},
				highlighter: {
					sizeAdjust: 0,
					tooltipSeparator: '&nbsp;'
				}
			});
			if (!expand)
				$('.jqplot-axis.jqplot-xaxis', $convas).remove();
				
			if (axisOnly)
				$('.jqplot-grid-canvas', $convas).remove();
		};
		
		// Main code //
		
		function initialDraw() {
			// copy DOM
			scheduleEditorId = ++scheduleEditorIds;
			$schedEditor.empty().append($('#scheduleEditor').clone().attr('id', 'schedEditor' + scheduleEditorId).show());
			$schedEditor.find('.dayGraph').each(function () {
				$(this).attr('id', $(this).attr('id') + '_' + scheduleEditorId);
			});
			// fill temperatures for each switch point
			$schedEditor.find('.switchPointTemp').each(function() {
				$(this).append($('#scheduleTemperatures').find('option').clone());
			});
			renderWdayGraph(null, null, null, 'left');
			for (var wd = 1; wd <= 7; wd++) {
				renderWdayGraph(wd, false);
				$schedEditor.find(wDayGraphId(wd)).attr('expand', '0').bind('click', function() {
					collapseExpand(parseInt($(this).attr('day'), 10));
				});
			};
			renderWdayGraph(null, null, null, 'right');
		};
		
		
		initialDraw();
		fillSwitchPoints(null);
		$schedEditor.find('#dayTemperatureDown').bind('click', function() {
			var wd = parseInt($(this).closest('#scheduleControl').find('#switchPoints').attr('day'), 10)
			var minMax = getMinMax(wd);
			if (minMax[0] <= TEMP_MIN)
				return;
			dayTemperatureShift(wd, -1);
			updateInterfaceOnChange(wd, undefined, true);
			fillSwitchPoints(wd)
		});
		$schedEditor.find('#dayTemperatureUp').bind('click', function() {
			var wd = parseInt($(this).closest('#scheduleControl').find('#switchPoints').attr('day'), 10)
			var minMax = getMinMax(wd);
			if (minMax[1] >= TEMP_MAX)
				return;
			dayTemperatureShift(wd, +1);
			updateInterfaceOnChange(wd, undefined, true);
			fillSwitchPoints(wd);
		});
		$schedEditor.find('#weekTemperatureDown').bind('click', function() {
			var wd = parseInt($(this).closest('#scheduleControl').find('#switchPoints').attr('day'), 10)
			var minMax = getMinMax();
			if (minMax[0] <= TEMP_MIN)
				return;
			for (var wdi = 1; wdi <= 7; wdi++) {
				dayTemperatureShift(wdi, -1);
				renderWdayGraph(wdi, wdi == wd);
			};
			fillSwitchPoints(wd);
		});
		$schedEditor.find('#weekTemperatureUp').bind('click', function() {
			var wd = parseInt($(this).closest('#scheduleControl').find('#switchPoints').attr('day'), 10)
			var minMax = getMinMax();
			if (minMax[1] >= TEMP_MAX)
				return;
			for (var wdi = 1; wdi <= 7; wdi++) {
				dayTemperatureShift(wdi, +1);
				renderWdayGraph(wdi, wdi == wd);
			};
			fillSwitchPoints(wd);
		});

		$schedEditor.find('#copyToDay').jeegoocontext('copyToSelector', {
			onSelect: function(event,context) {
				var fromDay = parseInt($(this).attr('fromDay'), 10);
				var toDay = parseInt($(this).attr('toDay'), 10);
				var wd = parseInt($schedEditor.find('#switchPoints').attr('day'), 10);
				for (var wdi = fromDay; wdi <= toDay; wdi++)
					copySchdule(wd, wdi);
			}
		}).bind('mousedown',function(event){ event.type='contextmenu'; $(this).trigger(event) });
	};
})(jQuery);
